import {
  Metadata, Loader, IEventParam, Http, IDictionary, Datasource,
} from '@zeedhi/core';

import {
  Select, Grid, Tabs, Form, Card, ModalService, Modal, Button, LoadingService,
} from '@zeedhi/common';

export class Pme00500ComensaisEstimadosController {
  private utilities = Loader.getInstance('Utilities');
  
  public appController = Loader.getInstance('AppController');
  
  public gridEditController = Loader.getInstance('GridEditableController');
  
  public reportController = Loader.getInstance('ReportController');
  
  public atualizarComensController = Loader.getInstance('Pme00510AtulizarComensaisCardController');
  
  public disabledButtons = true;
  
  public editable = false;
  
  public textDsTipoServ = '';
  
  private modalReport!: Modal;
  
  private armazenaForm: IDictionary = {};
  
  public setFilterUnidade({ component }: IEventParam<Select>) {
    const card: Card = Metadata.getInstance('formComensaisUnAgreg-card');
    
    const filter = [
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: this.appController.getCdCentral(),
      },
    ];
    
    component.datasource.filter = filter;
    component.datasource.get();
    card.isVisible = false;
  }
  
  public setGridComensaisUnAgreg() {
    const unidade = Metadata.getInstance<Select>('CDFILIAL');
    const grid: Grid = Metadata.getInstance('gridComensaisUnAgreg');
    
    const filter = [
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: this.appController.getCdCentral(),
      },
      {
        name: 'CDFILIAL',
        operator: '=',
        value: unidade.value ? unidade.value : '',
      },
    ];
    
    grid.datasource.filter = filter;
    grid.datasource.get();
  }
  
  public onChangeGridComensaisCurrentRow({ component }: IEventParam<Datasource>) {
    const form: Form = Metadata.getInstance('formComensaisUnAgreg');
    const row = component.currentRow;
    this.textDsTipoServ = row.DSTIPOSERV;
    this.disabledButtons = true;
    Http.post('/comensaisUnAgreg/dadosComensais', { requestType: 'Row', row }).then((response) => {
      const result = response.data.dataset?.result[0];
      if (result && response.data.dataset.result.length !== 0) {
        Object.keys(result).forEach((item) => {
          if (item in form.value) {
            form.value[item] = result[item];
          }
        });
        form.value.__is_new = false;
      } else {
        Object.keys(form.value).forEach((key) => {
          if (key.includes('QT')) {
            form.value[key] = 0;
          }
        });
        form.value.__is_new = true;
      }
      this.armazenaForm = JSON.parse(JSON.stringify(form.value));
    });
  }
  
  public cardIsVisible() {
    const card: Card = Metadata.getInstance('formComensaisUnAgreg-card');
    card.isVisible = true;
    const element = document.getElementsByName('formComensaisUnAgreg')[0];
    
    setTimeout(() => {
      element.scrollIntoView();
    }, 1);
  }
  
  private callLoading() {
    LoadingService.show(' ');
    window.setTimeout(() => LoadingService.hide(), 1000);
    window.setTimeout(() => this.utilities.showAlert('alert', 'pme::salvoComSucesso', 'success'), 1100);
    window.setTimeout(() => this.disableButtonSave(), 1200);
  }
  
  public disableButtonSave() {
    this.disabledButtons = true;
  }

  public saveForm() {
    const form: Form = Metadata.getInstance('formComensaisUnAgreg');
    const grid: Grid = Metadata.getInstance('gridComensaisUnAgreg');
    const rowGrid = grid.datasource.currentRow;
    const row = form.value;
    if (form.validate()) {
      if (row.EDITED) {
        (document.activeElement as HTMLElement).blur();
        if (row.CDFILCENTPME === '') {
          row.CDFILCENTPME = rowGrid.CDFILCENTPME;
          row.CDFILIAL = rowGrid.CDFILIAL;
          row.CDSERVFILI = rowGrid.CDSERVFILI;
          row.CDTIPOCARD = rowGrid.CDTIPOCARD;
        }
        
        this.callLoading();
        Http.post('/comensaisUnAgreg/save', { requestType: 'Row', row }).then((response) => {
          if (response.data.dataset.save[0]) {
            grid.datasource.get().then(() => {
              grid.datasource.currentRow = rowGrid;
              row.EDITED = false;
              form.value.__is_new = false;
              this.armazenaForm = JSON.parse(JSON.stringify(form.value));
            });
          }
        });
      } else this.utilities.showAlert('Alert', 'pme::nenhumRegEditado', 'warning');
    }
  }

  public cancelForm() {
    const form: Form = Metadata.getInstance('formComensaisUnAgreg');

    const valueForm = Object.entries(form.value);
    const valueConst = Object.entries(this.armazenaForm);
 
    const validate = (valueForm.length === valueConst.length) && valueForm.every((element, index) => element === valueConst[index]);
    this.disabledButtons = true;
    if (validate === false) {
      Object.keys(this.armazenaForm).forEach((item: string) => {
        form.value[item] = this.armazenaForm[item];
      });
    }
  }
  
  public setValuesTotal() {
    const form: Form = Metadata.getInstance('formComensaisUnAgreg');
    const valueForm = form.value;
    this.disabledButtons = false;
    valueForm.EDITED = true;
    valueForm.QTCOMPMESEG = valueForm.QTCLIPMESEG + valueForm.QTFUNPMESEG + valueForm.QTDOAPMESEG;
    valueForm.QTCOMPMETER = valueForm.QTCLIPMETER + valueForm.QTFUNPMETER + valueForm.QTDOAPMETER;
    valueForm.QTCOMPMEQUA = valueForm.QTCLIPMEQUA + valueForm.QTFUNPMEQUA + valueForm.QTDOAPMEQUA;
    valueForm.QTCOMPMEQUI = valueForm.QTCLIPMEQUI + valueForm.QTFUNPMEQUI + valueForm.QTDOAPMEQUI;
    valueForm.QTCOMPMESEX = valueForm.QTCLIPMESEX + valueForm.QTFUNPMESEX + valueForm.QTDOAPMESEX;
    valueForm.QTCOMPMESAB = valueForm.QTCLIPMESAB + valueForm.QTFUNPMESAB + valueForm.QTDOAPMESAB;
    valueForm.QTCOMPMEDOM = valueForm.QTCLIPMEDOM + valueForm.QTFUNPMEDOM + valueForm.QTDOAPMEDOM;
    valueForm.QTCOMPMEFER = valueForm.QTCLIPMEFER + valueForm.QTFUNPMEFER + valueForm.QTDOAPMEFER;
  }

  // Funções segunda tab.
  public filterGrids({ component }: IEventParam<Tabs>) {
    if (component.activeTab === 1) this.filterGridConsolidado();
  }
  
  public filterGridConsolidado() {
    const grid: Grid = Metadata.getInstance('gridConsolidadoCentral');
    
    const filter = [
      {
        name: 'P_CDFILCENTPME',
        operator: '=',
        value: this.appController.getCdCentral(),
      },
    ];
    
    grid.datasource.filter = filter;
    grid.datasource.get();
  }
  
  public onChangeGridConsolidadoCurrentRow({ component }: IEventParam<Grid>) {
    const form: Form = Metadata.getInstance('formConsolidadoCentral');
    const card: Card = Metadata.getInstance('formConsolidadoCentral-card');
    const row = component.datasource.currentRow;
    Http.post('/consolidadoCentral/dadosConsolidado', { requestType: 'Row', row }).then((response) => {
      const result = response.data.dataset.result[0];
      if (response.data.dataset.result.length !== 0) {
        Object.keys(result).forEach((item) => {
          if (item in form.value) {
            form.value[item] = result[item];
          }
        });
      }
      this.armazenaForm = JSON.parse(JSON.stringify(form.value));
    });
    card.isVisible = true;
  }

  public closeCardComensais() {
    const card: Card = Metadata.getInstance('formComensaisUnAgreg-card');
    card.isVisible = false;
  }
  
  public destroyModal() {
    if (this.modalReport) this.modalReport.destroy();
    this.atualizarComensController.destroyPeriodoModal();
  }
  
  // Modal Relatório
  public async createModalReport() {
    if (!this.modalReport) this.modalReport = await ModalService.createFromJson('modals/pme00520_relatorio_comensais_estimados', true);
  }
  
  public openModalReport() {
    this.createModalReport().then(() => {
      this.modalReport.show();
    });
  }
  
  public cancelModalReport() {
    this.modalReport.hide();
  }
  
  public clearFields({ component }: IEventParam<Select>) {
    const cardapio = Metadata.getInstance<Select>('TIPOCARD');
    const servico = Metadata.getInstance<Select>('SERVICO');
    
    if (component.name === 'CENTRAL') cardapio.setValue('');
    
    servico.setValue('');
  }
  
  public setFilterTpCardapio({ component }: IEventParam<Select>) {
    const form = Metadata.getInstance<Form>('formRelatorioComenEst');
    const filter: IDictionary = [];
    this.utilities.onMountedFilterFind(filter, 'CDFILCENTPME', '=', form.value.CENTRAL.CDFILIAL);
    this.utilities.onMountedFilterFind(filter, 'NRPASTATCARD', '=', null);
    
    component.datasource.filter = filter;
    component.datasource.get();
  }
  
  public setFilterServ({ component }: IEventParam<Select>) {
    const formComenEst = Metadata.getInstance<Form>('formRelatorioComenEst');
    const filter = [
      {
        name: 'P_CDFILIAL',
        operator: '=',
        value: formComenEst.value.CENTRAL.CDFILIAL,
      },
    ];
    
    component.datasource.filter = filter;
    component.datasource.get();
  }
  
  public setFilterAgreg({ component }: IEventParam<Select>) {
    const form = Metadata.getInstance<Form>('formRelatorioComenEst');
    const conditions = [
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: form.value.CENTRAL.CDFILIAL,
      },
      {
        name: 'CDGRUPFILI',
        operator: '=',
        value: 'T',
      },
    ];
    
    component.datasource.filter = conditions;
    component.datasource.get();
  }
  
  public clearFieldsComenEst({ component }: IEventParam<Select>) {
    const cardapio = Metadata.getInstance<Select>('TIPOCARD');
    const servico = Metadata.getInstance<Select>('SERVICO');
    const unidade = Metadata.getInstance<Select>('UNIDADE');
    
    if (component.name === 'CENTRAL') {
      cardapio.setValue('');
      servico.setValue('');
      unidade.setValue('');
    }
    
    if (component.name === 'TIPOCARD') {
      servico.setValue('');
      unidade.setValue('');
    }
  }
  
  public generateReport({ component }: IEventParam<Button>) {
    const form = Metadata.getInstance<Form>('formRelatorioComenEst');

    if (form.validate()) {
      const row: IDictionary = form.value;
      
      this.setParams(row);
      row.reportFormat = component.name;
      this.utilities.activeLoadingButton(component);
      this.reportController.generateReport('report/relatorioComensaisEstimados', row, row.CD_REPORT_CODE).finally(() => {
        this.utilities.disableLoadingButton(component);
      });
    }
  }
  
  private setParams(row: IDictionary) {
    const form = Metadata.getInstance<Form>('formRelatorioComenEst');
    row.REPORT_NAME = row.REPORT.REPORT_NAME;
    row.CD_REPORT_CODE = row.REPORT.CD_REPORT_CODE;
    row.NMFILIAL = form.value.CENTRAL.NMFILIAL;
  }
  
  public setValueCentral() {
    const selectCentral = Metadata.getInstance<Select>('CENTRAL');

    if (selectCentral.name === 'CENTRAL') {
      selectCentral.datasource.get().then((response: IDictionary) => {
        if (response.length > 0) {
          const defaultCentral = response.filter((row: IDictionary) => row.CDFILIAL === this.appController.getCdCentral());
          if (defaultCentral.length > 0) selectCentral.setValue(defaultCentral[0]);
        }
      });
    }
  }
}
